import matplotlib.pyplot as plt
import numpy as np


def binent(q):
 
    entropy=-q*np.log2(q)-(1-q)*np.log2(1-q)
    return entropy

    
def Q(mu,etad,t,pd):
 
    fQ=1.0/(2+mu*t*etad/2*pd) 
    return fQ

# Dark counts
pd= 5*1e-5# 
pdn=1-pd
#Bob's detector
etad=0.10

plt.figure(figsize=(3.375,2.5))
plt.rc('text', usetex=True)
plt.rc('text.latex', preamble=r'\usepackage{amsmath}  \usepackage{SIunits} ')


l=np.arange(0,90,0.0001);
 
alpha=0.2
t=10**(-alpha*l*0.1)
mu=t  # Optimal amplitude

#For V=1 D1=0  to avoid certain problems with infinities/zeros V=0.99
V=0.99
Q=0.5-V/2/(1+2*pd/mu/t/etad)



p3=1-np.exp(-mu)-np.exp(-mu)*mu-0.5*np.exp(-mu)*mu**2   #minus 0, 1 and 2 photon terms
pk1=0.5+0.5*np.sqrt(1-0.5)
q=0  #No bit flip for Alice
Is1=1-binent(pk1)


labels=['No injection',r'$4~ \micro\joule$',r'$6~ \micro\joule$',r'$10~ \micro\joule$']
colors=['r','g','b','k'] #'m',
styles=['-','--','-','-','--']
line=[[100,1],[5,3],[4,2],[3,1]] #,[8,2]]

Pref=17.2*1e-3;
mult=[0,200/Pref,300/Pref,500/Pref]




for x in range(0,4):
   muinj=3.32*10**(-6.3)*mult[x]
   mureal=mu+muinj

   D1=(1-V)/(2-mureal/t)
   I1D1=1-binent(0.5+np.sqrt(D1*(1-D1)))
   IAB=0.5*(mureal*t*etad+2*pd)*(1-binent(Q))
   IAE=0.5*mu*etad*((t-0.5*mureal)*I1D1+0.5*mureal)  # for Poissonian light g2=1   


   R=IAB-IAE
   #IAEr=etad*0.25*(mureal*t*Is1+0.5*p3*(1-Is1))
   #IAB=(mureal*t*etad*0.25)


   #IAE=etad*0.25*(mu*t*Is1+0.5*p3*(1-Is1)) #  Using Branciard, Gisin, Kraus and Scarani. 
 
#   print colors[x],styles[x],labels[x]
   l1,  =plt.semilogy(l,R,colors[x], dashes=[line[x][0],line[x][1]],label=labels[x])
#   l2,  =plt.semilogy( l,IAB,'c',linestyle=styles[x],label=labelsAB[x])



legend = plt.legend(loc='upper right', shadow=True,fontsize=6,handlelength=3)
fig = plt.gcf() 


plt.grid(True,which='both',color='darkgrey', linestyle='-', linewidth=0.5)
plt.ylabel(r'$R$',fontsize=8,rotation=0,labelpad=8)
plt.xlabel('Distance (km)',fontsize=8)
plt.grid(b=True, which='minor', linestyle='--')
plt.grid(b=True, which='mayor', linestyle='-')

ax = fig.add_subplot(111)
# We change the fontsize of minor ticks label 
plt.tick_params(axis='both', which='major', labelsize=6)

plt.xlim([0,70])
plt.ylim([1e-7,1e-1])

plt.subplots_adjust(bottom=0.14)   
plt.subplots_adjust(left=0.14)   
fig.savefig('RBB84.pdf', dpi=600) 


