import matplotlib.pyplot as plt
import numpy as np


def binent(q):         # Binary entropy function
 
    entropy=-np.multiply(q,np.log2(q))-np.multiply((1-q),np.log2(1-q))
    return entropy


# Dark counts
pd= 1e-5#2.63*1e-4
pdn=1-pd
#Bob's detector
etad=0.10

plt.figure(figsize=(3.375,2.5))
l=np.arange(0,100,0.001);          # Distance (in km)
 
alpha=0.2                          # Fiber attenuation (dB/km) 
t=(10**(-alpha*l*0.1)).reshape( len(10**(-alpha*l*0.1)  ), -1 )  # Channel transmission (size adjusted to be used in optimization loop)
 
#For V=1 D1=0  to avoid certain problems with infinities/zeros V=0.99
V=0.99


#labels=['No injection','200 W','300 W','500 W' ]   #Corresponding power for a CW laser
labels=['No injection',r'$4~ \micro\joule$',r'$6~ \micro\joule$',r'$10~ \micro\joule$'] # Energy in each 20 ns pulse
colors=['r','g','b','k'] #'m',
styles=['-','--','-','-','--']
line=[[100,1],[5,3],[4,2],[3,1]] #,[8,2]]
 
Pref=17.2*1e-3;            # Reference power from our experiment
mult=[0,200/Pref,300/Pref,500/Pref]  # Multiplication factor with respect to the reference.
 

mu=[]

# Optimization. Instead of using mu=t (approximation) we explore by brute force the possible parameters and take the average photon number mu giving the best rate.
for at in t:
   murange=np.arange(at*0.5,1.5*at,0.0001);
   
   D1=(1-V)/(2-murange/at)                                    # Eq. (25) in PRA 71, 042316 (2005)              
   I1D1=1-binent(0.5+np.sqrt(D1*(1-D1)))                      # Eq. (13) in PRA 71, 042316 (2005)
   Q=0.5-V/2/(1+2*pd/murange/at/etad)                         # Eq. (23) in PRA 71, 042316 (2005)

   IAB=0.5*(murange*at*etad+2*pd)*(1-binent(Q))               # Eq. (22) in PRA 71, 042316 (2005)
   IAE=0.5*murange*etad*((at-0.5*murange)*I1D1+0.5*murange)   # Eq. (26) in PRA 71, 042316 (2005) for Poissonian light g2=1 
 
   R=IAB-IAE

   muopt=murange[np.where(R == np.amax(R))[0]]                # Take mu corresponding to the best key rate
 
   mu.append(muopt)

#mu is a list with the optimal mean photon numbers for each channel transmission

# Using the optimal values:

Q=0.5-V/2/(1+2*pd/np.divide(np.array(mu),t)/etad) 


for x in range(0,4):

   muinj=3.32*10**(-6.3)*mult[x]
   mureal=np.array(mu)+muinj                                  # Actual number of photons after the injection
 
   divis=[]
   for i in range(t.size):
      divis.append(mureal[i]/t[i])
   
   D1=(1-V)/(2-np.array(divis))                               # Eq. (25) in PRA 71, 042316 (2005)   
   

   I1D1=1-binent(0.5+np.sqrt(np.multiply(D1,(1-D1))))        # Eq. (13) in PRA 71, 042316 (2005)
   IAB=np.multiply(0.5*(np.multiply(mureal,t)*etad+2*pd),(1-binent(Q)))  # Eq. (22) in PRA 71, 042316 (2005)
   

   IAE=0.5*np.multiply(mureal,etad*(np.multiply((t-0.5*mureal),I1D1)+0.5*mureal))  # Eq. (26) in PRA 71, 042316 (2005) for Poissonian light g2=1 

   R=IAB-IAE
 
   l1,  =plt.semilogy(l,R,colors[x], dashes=[line[x][0],line[x][1]],label=labels[x])



# Plot the data and save to file
legend = plt.legend(loc='upper right', shadow=True,fontsize=6,handlelength=3)
fig = plt.gcf() 


plt.grid(True,which='both',color='darkgrey', linestyle='-', linewidth=0.5)
plt.ylabel(r'$R$',fontsize=8,rotation=0,labelpad=8)
plt.xlabel('Distance (km)',fontsize=8)
plt.grid(b=True, which='minor', linestyle='--')
plt.grid(b=True, which='mayor', linestyle='-')

ax = fig.add_subplot(111)
plt.tick_params(axis='both', which='major', labelsize=6)

plt.xlim([0,80])
plt.ylim([1e-7,1e-1])

plt.subplots_adjust(bottom=0.14)   
plt.subplots_adjust(left=0.14)   
fig.savefig('RBB84Opt.pdf', dpi=600) 


