import matplotlib.pyplot as plt
import numpy as np

def binent(q):          # Binary entropy function
 
    entropy=-q*np.log2(q)-(1-q)*np.log2(1-q)
    return entropy


plt.rc('text', usetex=True)
plt.rc('text.latex', preamble=r'\usepackage{amsmath}  \usepackage{SIunits} ')

plt.figure(figsize=(3.375,2.5))



# Dark counts
pd=1e-5
pdn=1-pd

#Bob's detector
etad=0.10


l=np.arange(15,120,0.01);  # Distance (in km)
 
alpha=0.2                  # Fiber attenuation (dB/km)
t=10**(-alpha*l*0.1)       # Channel transmission
mu=2*np.sqrt(t)            # Optimal amplitude

pk1=0.5+0.5*np.sqrt(1-0.5) # From the formulas in Section IV.C.2 of Branciard, Gisin, Kraus and Scarani: PRA 72, 032301 (2005)
q=0                        # No bit flip for Alice
Is1=1-binent(pk1)          # Eq. (57) in PRA 72, 032301 (2005)


#labels=['No injection','200 W','300 W','500 W' ]   #Corresponding power for a CW laser
labels=['No injection',r'$4~ \micro\joule$',r'$6~ \micro\joule$',r'$10~ \micro\joule$'] # Energy in each 20 ns pulse
colors=['r','g','b','k'] 
styles=['-','--','-','-','--']
line=[[100,1],[5,3],[4,2],[3,1]] 
 
Pref=17.2*1e-3;            # Reference power from our experiment
mult=[0,200/Pref,300/Pref,500/Pref]  # Multiplication factor with respect to the reference.
 


for x in range(0,4):
   muinj=3.32*10**(-3)*mult[x]*10**(-3.15)*10**(-0.25*np.log10(t))
   mureal=mu+muinj                           # Actual number of photons after the injection                 

   R=etad/4*(1-Is1)*(mureal*t-mureal**3/12)  # Eq. (90) in PRA 72, 032301 (2005) 
  
   print(colors[x],styles[x],labels[x])
   l1,  =plt.semilogy( l,R,colors[x], dashes=[line[x][0],line[x][1]],label=labels[x])   


# Figure generation
legend = plt.legend(loc='upper right', shadow=True,fontsize=6,handlelength=3)
fig = plt.gcf() 


plt.grid(True,which='both',color='darkgrey', linestyle='-', linewidth=0.5)
plt.ylabel(r'$R$',fontsize=8,rotation=0,labelpad=8)
plt.xlabel('Distance (km)',fontsize=8)
plt.grid(b=True, which='minor', linestyle='--')
plt.grid(b=True, which='mayor', linestyle='-')

ax = fig.add_subplot(111)
# We change the fontsize of minor ticks label 
plt.tick_params(axis='both', which='major', labelsize=6)

plt.xlim([15,120])
plt.ylim([1e-7,1e-2])

plt.subplots_adjust(bottom=0.14)   
plt.subplots_adjust(left=0.14)   
fig.savefig('RSARG04.pdf', dpi=600) 

